{###############################################################################}
{##}
{#   Copyright (C) <2016>  <Florian Alu - Prolibre - http://prolibre.com>#}
{##}
{#    This program is free software: you can redistribute it and/or modify#}
{#    it under the terms of the GNU Affero General Public License as published#}
{#    by the Free Software Foundation, either version 3 of the License.#}
{#    #}
{##}
{#    This program is distributed in the hope that it will be useful,#}
{#    but WITHOUT ANY WARRANTY; without even the implied warranty of#}
{#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the#}
{#    GNU Affero General Public License for more details.#}
{##}
{#    You should have received a copy of the GNU Affero General Public License#}
{#    along with this program.  If not, see <http://www.gnu.org/licenses/>.#}
{##}
{###############################################################################}

{% if field.is_hidden %}
    {{ field }}
{% else %}
    <div id="div_{{ field.auto_id }}" class="form-group{% if wrapper_class %} {{ wrapper_class }}{% endif %}{% if form_show_errors and field.errors %} has-error{% endif %}{% if field.css_classes %} {{ field.css_classes }}{% endif %}">

        {% if field.label %}
            <label for="{{ field.auto_id }}"  class="control-label {{ label_class }}{% if field.field.required %} requiredField{% endif %}">
                {{ field.label|safe }}{% if field.field.required %}<span class="asteriskField">*</span>{% endif %}
            </label>
        {% endif %}

        {% include 'layout/radioselect_image.html' %}
    </div>
{% endif %}
